/*-
 * Copyright (C)2023..2024 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: pgoblin5pgeo.h,v 1.13 2024/07/21 20:25:08 babolo Exp $
 */

#ifndef _PGOBLIN_5_PGEO_H_
# define _PGOBLIN_5_PGEO_H_

__BEGIN_DECLS

# ifndef PGOBLIN_COMPAT
#  warning PGOBLIN_COMPAT not defined
# elif PGOBLIN_COMPAT < 5
#  error pgoblin5 is not ABI compatible with old versions
# endif

# define PGOBLIN_STYLE_ID     "#pGoblin-pgeo"
# define PGOBLIN_SZLEV1       4
# define PGOBLIN_SZLEV2       4
# define PGOBLIN_SZLEV3       8

# define PGOBLIN_GEO_POINT    1U
# define PGOBLIN_GEO_LINE     2U
# define PGOBLIN_GEO_LSEG     3U
# define PGOBLIN_GEO_BOX      4U
# define PGOBLIN_GEO_CIRCLE   5U
# define PGOBLIN_GEO_PATH     6U
# define PGOBLIN_GEO_POLYGON  7U
# define PGOBLIN_GEO_PATHP    8U
# define PGOBLIN_GEO_POLYP    9U
# define PGOBLIN_GEO_MAXGEO   7U
# define PGOBLIN_GEO_MAXCOLOR 9U

struct confchain;
typedef struct confchain confchain;
struct confchain {
    confchain  *next ;
    char       *name ; /*          */
    u_int64_t   fg   ; /*             */
    int         col  ; /*        */
    u_int16_t   chn  ; /*    */
    u_int8_t    type ; /*       */
    int8_t      dummy;
};

typedef struct coname {
    u_int64_t   fg   ; /*             */
    char       *name ; /*          */
} coname;

typedef struct confar {
    u_int64_t   fg    ;   /*                         */
    union {
        coname    *namec; /*                 */
        confchain *names; /*     */
    };
    int         nmsz  ;   /*            */
    u_int8_t    def   ;   /*                      */
    int8_t      dummy0;
    int16_t     dummy1;
} confar;

struct pgoblin_realsty {
    char        id[PGOBLIN_STRING_ID_LEN];    /*  "#pGoblin"           */
    union {
        struct {
            BLIN_flag dummy0   : 8
                    , format   : 4
                    , dummy1   : 4
                    , solid    : 1
                    , dummy3   : 7
                    , blin     : 8
                ;
            };
        BLIN_flag   flags;
    };
#   define PGOBLIN_LONG      0x1U
#   define PGOBLIN_ALFA      0x2U
#   define PGOBLIN_COLOR     0x4U
#   define PGOBLIN_BIT       0x8U
#   define PGOBLIN_NOFORM    0xEU
/*  : 0 -- 1           */
/*                     1 -- 2          */
/*                     2 -- 2  +  */
/*                     3 -- 4  +  */
/*                     4 -- 4 RGB +    */
/*                     5 -- 8 RGB +     */
/*                     6 -- 4 RGB +    */
/*                     7 -- 8 RGB +     */
/*                     8 -- 1                 */
    u_int16_t    wideline;
    u_int16_t    dummy;
    u_int64_t    bg;
    u_int64_t    fg0;
    confar      *fg[PGOBLIN_GEO_MAXCOLOR + 1];
    int          sz[PGOBLIN_GEO_MAXCOLOR + 1];
    double       scale[4];
#   define   scalexup    scale[0]
#   define   scalexdown  scale[1]
#   define   scaleyup    scale[2]
#   define   scaleydown  scale[3]
    int          margin[4];
#   define   marginxdown margin[0]
#   define   marginxup   margin[1]
#   define   marginydown margin[2]
#   define   marginyup   margin[3]
    int          fill[4];
#   define   fillxdown   fill[0]
#   define   fillxup     fill[1]
#   define   fillydown   fill[2]
#   define   fillyup     fill[3]
    void       **grole[1 << (PGOBLIN_SZLEV1 * 2)];
};

extern pgoblin_styles PGOBLIN_MODULE;

__END_DECLS

#endif /* !_PGOBLIN_5_PGEO_H_ */
